/*
 * LaTeX.c
 * gaim-latex plugin
 *
 * This a plugin for GAIM to display LaTeX formula in conversation
 *
 * PLEASE , send any comment , bug report , greetz, death threat to
 * 		Nicolas Schoonbroodt <nicolas@ffsa.be>
 *
 * Copyright (C) 2004 Nicolas Schoonbroodt (nicolas@ffsa.be)
 * some portions : Copyright (C) 2004 GRIm@ (thegrima@altern.org).
 * 				  Copyright (C) 2004 Eric Betts (bettse@onid.orst.edu).
 * Other portions heavily inspired and copied from gaim sources
 * Copyright (C) 1998-2004 Gaim developers gaim.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; This document is under the scope of
 * the version 2 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see COPYING); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "LaTeX.h"


/* Verify Blacklist */
gboolean is_blacklisted(char *message)
{
  char *not_secure[NB_BLACKLIST] = BLACKLIST;
  int i;
  for (i = 0 ; i < NB_BLACKLIST ; i++)
    {
      if (strstr(message, not_secure[i]) != NULL) return TRUE;
    }
  return FALSE;
}

/* 
 * Transform latex in a png file in FILE_PNG
 */
gboolean latex_to_image(char *latex)
{
  FILE * texfile;
  
  if (! (texfile = fopen(FILE_TEX, "w"))) return FALSE;
  fprintf(texfile, HEADER HEADER_MATH "%s" FOOTER_MATH FOOTER, latex);
  fclose (texfile);

  if (chdir("/tmp")) return FALSE;
  if ( system("latex -interaction=nonstopmode " FILE_TEX) || system("dvips -o" FILE_PS " -E " FILE_DVI) ||  system("convert " FILE_PS " " FILE_PNG) ) return FALSE;
  return TRUE;
}


/*  
 * Transform *tmp2 extracting some[tex]here[/tex]thing, made LaTeX image
 *  And tmp2 become 'some<img="number">thing'
 *  t[] and i are here to remember image id, for freeing it
 */
static gboolean analyse(char **tmp2, int t[], int *i)
{
  int pos1, pos2, idimg;
  char *ptr1, *ptr2;
  FILE *fi;
  gboolean retro = TRUE;

  ptr1 = strstr(*tmp2, KOPETE_TEX);
	
  while(ptr1 != NULL)
    {
      retro = FALSE;
      char *tex, *message, *idstring;
      gchar *name, *buf;
      char *filedata;
      size_t size;
      GError *error = NULL;
      char *filter;

      pos1 = strlen(*tmp2) - strlen(ptr1);
		
      // Have to ignore the first 2 char ("$$") --> & [2]
      ptr2 = strstr(&ptr1[2], KOPETE_TEX);

      if (ptr2 == NULL)
	{ return FALSE; }

      pos2 = strlen(*tmp2) - strlen(ptr2) + 2;

      if ((tex = malloc(pos2 - pos1 - 3)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      strncpy(tex, &ptr1[2], pos2 - pos1 - 4);
      tex[pos2-pos1-4] = '\0';

      // Gaim transform & to &amp; and I make the inverse transformation
      while ( (filter = strstr(tex, FILTER_AND) ) != NULL)
	{
	  strcpy(&tex[strlen(tex) - strlen(filter) + 1], &filter[5]); 
	}
      // <br> filter
      while ( (filter = strstr(tex, FILTER_BR) ) != NULL)
	{
	  strcpy(&tex[strlen(tex) - strlen(filter)], &filter[4]);
	}

      // Creates the image in FILE_PNG
      if (!latex_to_image(tex)) { free(tex); return FALSE; };
      free(tex);

      // loading image
      if (!g_file_get_contents(FILE_PNG, &filedata, &size, &error))
	{
	  gaim_notify_error(NULL, NULL, error->message, NULL);
	  g_error_free(error);
	  return FALSE;
	}
	
      system("rm /tmp/gaimTeX.*");

      name = "gaimTeX.png";
      idimg = gaim_imgstore_add(filedata, size, name);
      g_free(filedata);
      filedata = NULL;

      if (idimg == 0)
	{
	  buf = g_strdup_printf(_("Failed to store image: %s\n"), FILE_PNG);
	  gaim_notify_error(NULL,NULL, buf, NULL);
	  g_free(buf);
	  return FALSE;
	}


      if((message = malloc(strlen(*tmp2)+1)) == NULL)
	{
	  // TODO: Notify Error
	  return FALSE;
	}
		
      strcpy(message,*tmp2);


      idstring = malloc(10);
      sprintf(idstring, "%d", idimg);

      // making new message
      if ((message = malloc (strlen(*tmp2) - pos2 + pos1 + strlen(idstring) + strlen(IMG_BEGIN) + strlen(IMG_END) + 1)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      if (pos1 > 0)
	{
	  strncpy(message, *tmp2, pos1);
	  message[pos1] = '\0';
	  strcat(message, IMG_BEGIN);
	} else  {
	  strcpy(message, IMG_BEGIN);
	}

      strcat(message, idstring);
      strcat(message, IMG_END);

      if (pos2 < strlen(*tmp2))
	{ strcat(message, &ptr2[2]); }

      free(*tmp2);
      if ((*tmp2 = malloc(strlen(message)+1)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      strcpy(*tmp2, message);
      free(message);
      free(idstring);

      t[*i] = idimg;
      *i = *i + 1;

      ptr1 = strstr(*tmp2, KOPETE_TEX);
    }

  /*
   * Retro compatible with gaim-latex <= 0.2, try to make it compatible with KopeteTeX, sot [tex] [/tex] no more used
   */
  if (!retro) return TRUE;

  ptr1 = strstr(*tmp2, BEG);
	
  while(ptr1 != NULL)
    {
      char *tex, *message, *idstring;
      gchar *name, *buf;
      char *filedata;
      size_t size;
      GError *error = NULL;
      char *filter;

      pos1 = strlen(*tmp2) - strlen(ptr1);
      ptr2 = strstr(ptr1, END);

      if (ptr2 == NULL)
	{ return FALSE; }

      pos2 = strlen(*tmp2) - strlen(ptr2) + 6;

      if ((tex = malloc(pos2 - pos1 - 10)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      strncpy(tex, &ptr1[5], pos2 - pos1 - 11);
      tex[pos2-pos1-11] = '\0';

      // Gaim transform & to &amp; and I make the inverse transformation
      while ( (filter = strstr(tex, FILTER_AND) ) != NULL)
	{
	  strcpy(&tex[strlen(tex) - strlen(filter) + 1], &filter[5]); 
	}

      // Creates the image in FILE_PNG
      if (!latex_to_image(tex)) { free(tex); return FALSE; };
      free(tex);

      // loading image
      if (!g_file_get_contents(FILE_PNG, &filedata, &size, &error))
	{
	  gaim_notify_error(NULL, NULL, error->message, NULL);
	  g_error_free(error);
	  return FALSE;
	}
	
      system("rm /tmp/gaimTeX.*");

      name = "gaimTeX.png";
      idimg = gaim_imgstore_add(filedata, size, name);
      g_free(filedata);
      filedata = NULL;

      if (idimg == 0)
	{
	  buf = g_strdup_printf(_("Failed to store image: %s\n"), FILE_PNG);
	  gaim_notify_error(NULL,NULL, buf, NULL);
	  g_free(buf);
	  return FALSE;
	}


      if((message = malloc(strlen(*tmp2)+1)) == NULL)
	{
	  // TODO: Notify Error
	  return FALSE;
	}
		
      strcpy(message,*tmp2);


      idstring = malloc(10);
      sprintf(idstring, "%d", idimg);

      // making new message
      if ((message = malloc (strlen(*tmp2) - pos2 + pos1 + strlen(idstring) + strlen(IMG_BEGIN) + strlen(IMG_END) + 1)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      if (pos1 > 0)
	{
	  strncpy(message, *tmp2, pos1);
	  message[pos1] = '\0';
	  strcat(message, IMG_BEGIN);
	} else  {
	  strcpy(message, IMG_BEGIN);
	}

      strcat(message, idstring);
      strcat(message, IMG_END);

      if (pos2 < strlen(*tmp2))
	{ strcat(message, &ptr2[6]); }

      free(*tmp2);
      if ((*tmp2 = malloc(strlen(message)+1)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      strcpy(*tmp2, message);
      free(message);
      free(idstring);

      t[*i] = idimg;
      *i = *i + 1;

      ptr1 = strstr(*tmp2, BEG);
    }
  /*
   * End of retro compatibility
   */
  return TRUE;
}

/*
 * gaim_latex_write perform the effective write of the latex code in the IM windows
 * 	*conv is a pointer onto the conversation context
 *	*nom is the name of the correspondent
 *	*message is the modified message with the image
 *	*messFlags is Flags related to the messages
 *	*original is the original message unmodified
 * return TRUE.
 */
static gboolean gaim_latex_write(GaimConversation *conv, char *nom, char *message, GaimMessageFlags messFlag, char *original, gboolean to_send)
{
  gboolean logflag;

      if (to_send) serv_send_im(gaim_conversation_get_gc(conv), gaim_conversation_get_name(conv), original, messFlag);

  // writing log
  logflag = gaim_conversation_is_logging(conv);

  if (logflag)
    {
      if (to_send) gaim_log_write(conv->log, GAIM_MESSAGE_SEND, gaim_account_get_alias(conv->account), time(NULL), original);
      else gaim_log_write(conv->log, GAIM_MESSAGE_RECV, nom, time(NULL), original);
      gaim_conversation_set_logging(conv,FALSE);
    }

  gaim_conv_im_write(gaim_conversation_get_im_data(conv), nom, message, messFlag, time(NULL));

  if (logflag)
    { gaim_conversation_set_logging(conv,TRUE); }

  return FALSE;
}

/*
 * gaim_latex_imgstore_unref
 * TODO : verify this... UGLY
 */
void gaim_latex_imgstore_unref(int **t)
{
  int i = 0, j;

  for (j=0; j<i;j = j+1)
    {
      gaim_imgstore_unref(*t[j]);
      gaim_imgstore_unref(*t[j]);
    }


  return;
}

/*
 * On sending a message
 */
static gboolean message_send(GaimAccount *account, GaimConversation *conv, char **buffer, void *data)
{
  char *tmp2;
  int t[10], i = 0;//, j;

  // if nothing to do
  if (strstr(*buffer, BEG) == NULL && strstr(*buffer,KOPETE_TEX) == NULL)
    {
      return FALSE;
    };

  if (is_blacklisted(*buffer)) return FALSE;

  if((tmp2 = malloc(strlen(*buffer)+1)) == NULL)
    {
      // TODO: Notify Error
      return FALSE;
    }

  strcpy(tmp2,*buffer);

  if (analyse(&tmp2, t, &i) )
    {
      char *name2;

      // TODO : THIS IS VERY VERY UGLY
      // TODO : MODIFY TO MINIMIZE CALL
      // finding name of sender
      if (gaim_account_get_alias(account) != NULL)
	{
	  name2 = malloc(strlen(gaim_account_get_alias(account))+1);
	  strcpy(name2, gaim_account_get_alias(account));
	} 
      else	if (gaim_account_get_username(account) != NULL)
	{
	  name2 = malloc(strlen(gaim_account_get_username(account))+1);
	  strcpy(name2,gaim_account_get_username(account));
	} else {
	  free(tmp2);
	  gaim_latex_imgstore_unref((int **) &t);	
	  return FALSE;
	}

      gaim_latex_write(conv, name2, tmp2, GAIM_MESSAGE_SEND, *buffer, TRUE);

      gaim_latex_imgstore_unref((int **) &t);	

      free(tmp2);
      free(name2);

      return TRUE;
    }

  free(tmp2);

  gaim_latex_imgstore_unref((int **) &t);	

  return FALSE;
}

/*
 * when receiving a message
 */
static gboolean message_recv(GaimAccount *account, char **sender, char **buffer, GaimMessageFlags *flags, void *data)
{

  GaimConversation *conv;

  // if no [tex] -> nothing to do
  if (strstr(*buffer, BEG) == NULL && strstr(*buffer, KOPETE_TEX) == NULL)
    {
      return FALSE;
    };

  if (is_blacklisted(*buffer)) return FALSE;

  if (gaim_conversation_get_im_data((conv = gaim_find_conversation_with_account(*sender, account))) != NULL)
    {
      char *tmp2;
      int t[10], i = 0;//, j;
	
      if ((tmp2 = malloc(strlen(*buffer)+1)) == NULL)
	{
	  // TODO: Report the error
	  return FALSE;
	}

      strcpy(tmp2,*buffer);

      if (analyse(&tmp2, t, &i));
      {
      	gaim_latex_write(conv, *sender, tmp2, GAIM_MESSAGE_RECV, *buffer, FALSE);

	gaim_latex_imgstore_unref((int **) &t);

	free(tmp2); tmp2 = NULL;
	free(*buffer);
	*buffer = NULL;
	return FALSE;
      }
      gaim_latex_imgstore_unref((int **) &t);

      free(tmp2);
      return FALSE;
    }

  return FALSE;
}


static gboolean plugin_load(GaimPlugin *plugin)
{
  void *conv_handle = gaim_conversations_get_handle();

  gaim_signal_connect(conv_handle, "writing-im-msg",
		      plugin, GAIM_CALLBACK(message_send), NULL);

  gaim_signal_connect_priority(conv_handle, "receiving-im-msg",
		      plugin, GAIM_CALLBACK(message_recv), NULL, -9999);
  gaim_debug(GAIM_DEBUG_INFO, "LaTeX", "LaTeX loaded\n");

  return TRUE;
}

gboolean plugin_unload(GaimPlugin * plugin)
{
  void *conv_handle = gaim_conversations_get_handle();

  gaim_signal_disconnect(conv_handle, "writing-im-msg", plugin, GAIM_CALLBACK(message_send));
  gaim_signal_disconnect(conv_handle, "receiving-im-msg", plugin, GAIM_CALLBACK(message_recv));

  return TRUE;
}


static GaimPluginInfo info =
  {
    GAIM_PLUGIN_MAGIC,
    GAIM_MAJOR_VERSION,
    GAIM_MINOR_VERSION,
    GAIM_PLUGIN_STANDARD,                             /**< type           */
    NULL,                                             /**< ui_requirement */
    0,                                                /**< flags          */
    NULL,                                             /**< dependencies   */
    GAIM_PRIORITY_DEFAULT,                            /**< priority       */

    LATEX_PLUGIN_ID,                                  /**< id             */
    N_("LaTeX"),                                      /**< name           */
    N_("0.3"),                                        /**< version        */
    /**  summary        */
    N_("To display LaTeX formula into Gaim conversation."),
    /**  description    */
    N_("Put your code between $$ ... $$ markup.\n[tex] ... [/tex] markup are now deprecated (for compatibility with Kopete)\nYou must have LaTeX and ImageMagick installed (in your PATH)"),
    "Nicolas Schoonbroodt <nicolas@ffsa.be>",         /**< author         */
    WEBSITE,                                          /**< homepage       */

    plugin_load,                                      /**< load           */
    plugin_unload,                                    /**< unload         */
    NULL,                                             /**< destroy        */
    NULL,                                             /**< ui_info        */
    NULL,                                             /**< extra_info     */
    NULL,
    NULL
  };



static void
init_plugin(GaimPlugin *plugin)
{
}

GAIM_INIT_PLUGIN(LaTeX, init_plugin, info)
