/*
 * LaTeX.c
 * gaim-latex plugin
 *
 * This a plugin for GAIM to display LaTeX formula in conversation
 *
 * PLEASE , send any comment , bug report , greetz, death threat to
 * 		Nicolas Schoonbroodt <nicolas@ffsa.be>
 *
 * Copyright (C) 2005 Nicolas Schoonbroodt (nicolas@ffsa.be)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; This document is under the scope of
 * the version 2 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see COPYING); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef _GAIM_LATEX_H_
#define _GAIM_LATEX_H_

//include
#include <stdio.h>
#include <string.h>

#include <gaim/internal.h>
#include <gaim/conversation.h>
#include <gaim/core.h>
#include <gaim/debug.h>
#include <gaim/signals.h>
#include <gaim/imgstore.h>
#include <gaim/util.h>
#include <gaim/notify.h>
#include <gaim/gtkimhtml.h>
#include <gaim/server.h>
#include <gaim/log.h>
#include <gaim/version.h>

// Constant
#define FILE_TEX "/tmp/gaimTeX.tex"
#define FILE_PNG "/tmp/gaimTeX.png"
#define FILE_DVI "/tmp/gaimTeX.dvi"
#define FILE_PS "/tmp/gaimTeX.ps"

#define IMG_BEGIN "<img id=\""
#define IMG_END "\">"

#define BEG "[tex]"
#define END "[/tex]"
#define KOPETE_TEX "$$"


#define LATEX_PLUGIN_ID "yoda222-LaTeX"
#define WEBSITE "http://sourceforge.net/projects/gaim-latex/"

#define HEADER "\\documentclass[12pt]{article}\\usepackage[dvips]{graphicx}\\pagestyle{empty}"
#define HEADER_MATH "\\begin{document}\\begin{eqnarray*}"

#define FOOTER "\\end{document}"
#define FOOTER_MATH "\\end{eqnarray*}"

#define FILTER_AND "&amp;"
#define FILTER_BR "<br>"

/* Yes, this is simply a copy/paste of KopeteTex blacklist. */
/* But to bad in LaTeX and system security for verify all   */
/* of this */
#define NB_BLACKLIST (42)
#define BLACKLIST {"\\def","\\let","\\futurelet","\\newcommand","\\renewcomment","\\else","\\fi","\\write","\\input","\\include","\\chardef","\\catcode","\\makeatletter","\\noexpand","\\toksdef","\\every","\\errhelp","\\errorstopmode","\\scrollmode","\\nonstopmode","\\batchmode","\\read","\\csname","\\newhelp","\\relax","\\afterground","\\afterassignment","\\expandafter","\\noexpand","\\special","\\command","\\loop","\\repeat","\\toks","\\output","\\line","\\mathcode","\\name","\\item","\\section","\\mbox","\\DeclareRobustCommand"}


// prototypes

/* Verify Blacklist */
/* return true if one word of the message is blacklisted */
gboolean is_blacklisted(char *message);

/* Create image in FILE_PNG with the LaTeX code pointed by *latex */
gboolean latex_to_image(char *latex);

/*  
 * Transform *tmp2 extracting some[tex]here[/tex]thing, made LaTeX image
 *  And tmp2 become 'some<img="number">thing'
 *  t[] and i are here to remember image id, for freeing it
 */
static gboolean analyse(char **tmp2, int t[], int *i);

/*
 * gaim_latex_write perform the effective write of the latex code in the IM windows
 * 	*conv is a pointer onto the conversation context
 *	*nom is the name of the correspondent
 *	*message is the modified message with the image
 *	*messFlags is Flags related to the messages
 *	*original is the original message unmodified
 * return TRUE.
 */
static gboolean gaim_latex_write(GaimConversation *conv, char *nom, char *message, GaimMessageFlags messFlag, char *original, gboolean to_send);

/*
 * gaim_latex_imgstore_unref
 * Free the created images
 * (not sure it's perfectly ok)
 */
void gaim_latex_imgstore_unref(int **t);

/* to intercept outgoing messages */
static gboolean message_send(GaimAccount *account, GaimConversation *conv, char **buffer, void *data);

/* to intercept incoming messages */
static gboolean message_recv(GaimAccount *account, char **sender, char **buffer, GaimMessageFlags *flags, void *data);

#endif
